// Copyright (c) 2015, Microsoft Corporation
//
// Permission to use, copy, modify, and/or distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
// SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
// IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//

package com.microsoft.alljoyn.minecraft;

import net.minecraft.client.Minecraft;
import net.minecraft.client.entity.EntityPlayerSP;
import net.minecraft.client.multiplayer.WorldClient;
import net.minecraft.util.BlockPos;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraftforge.event.terraingen.BiomeEvent.BiomeColor;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

public class BiomeEventHandler {

	private String biomeName = "";
	private short tempLevel = 0;
	
	private AllJoynACTemperatureClient allJoynACTemperatureClient = new AllJoynACTemperatureClient();	
	
	
    public BiomeEventHandler()
    {
    	allJoynACTemperatureClient.initialize();
    }
    
	///
	/// Called when entering a new biome
	///
    @SubscribeEvent
    public void onBiomeColor(BiomeColor event)
    {
    	WorldClient worldObj = Minecraft.getMinecraft().theWorld;
    	EntityPlayerSP thePlayer = Minecraft.getMinecraft().thePlayer;
    	
		int posX = (int)Math.floor(thePlayer.posX);
	    int posY = (int)Math.floor(thePlayer.posY);
	    int posZ = (int)Math.floor(thePlayer.posZ);
	    BlockPos blockpos = new BlockPos(posX, posY, posZ);
	    
	    BiomeGenBase biome = worldObj.getChunkFromBlockCoords(blockpos).getBiome(blockpos, worldObj.getWorldChunkManager());
    		    
        String bn = biome.biomeName;
        if (bn != this.biomeName) {
        	this.biomeName = bn;
        	System.out.println("BiomeName=" + this.biomeName + " Temp=" + biome.temperature);
        	
        	// Set A/C property from biome.temperature
        	short newTempLevel;
        	String tempLevelName;
        	if (biome.temperature < 0.2) {		// Snow, ice, frozen, cold
        		newTempLevel = -2;
        		tempLevelName = "Cold";
        	} else if (biome.temperature < 0.5) { // Cold, hills
        		newTempLevel = -1;
        		tempLevelName = "Cool";
        	} else if (biome.temperature < 0.8) { // Water, forest
        		newTempLevel = 0;
        		tempLevelName = "Comfortable";
        	} else if (biome.temperature < 1.0) { // Plains, island, jungle
        		newTempLevel = 1;
        		tempLevelName = "Warm";
        	} else {				 // Desert, savanna, mesa
        		newTempLevel = 2;
        		tempLevelName = "Hot";
        	}
        	
        	RenderEventHandler.drawText2("Temp: " + tempLevelName + " ");
        	if (newTempLevel != this.tempLevel) {
        		System.out.println(String.format("Set temperature level to %d", newTempLevel));
        		allJoynACTemperatureClient.incrementUInt16Property("Value", newTempLevel-this.tempLevel);       		
	        	this.tempLevel = newTempLevel;
        	}
        }
    }    
}
