// Copyright (c) 2015, Microsoft Corporation
//
// Permission to use, copy, modify, and/or distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
// SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
// IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//

package com.microsoft.alljoyn.minecraft;

import net.minecraft.client.Minecraft;
import net.minecraft.client.entity.EntityPlayerSP;
import net.minecraft.entity.Entity;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.TickEvent;
import net.minecraftforge.fml.common.gameevent.TickEvent.RenderTickEvent;

public class LightChangeEventHandler {

	private static long MAX_ELAPSE = 2000;
	private static long MIN_ELAPSE = 160;
	private static long LSF_SETPROP_DELAY = MIN_ELAPSE/2;

	private long elapse = MAX_ELAPSE;
	private long ms0 = 0;
	private AllJoynLSFClient allJoynLSFClient = new AllJoynLSFClient();	

	private RenderEventHandler renderer = null;
	private float [] colorHSB = new float [3];
	private static String [] stringHSB = new String[] {"Hue", "Saturation", "Brightness"};
	
	public static boolean refreshNeeded = false;
	
    public LightChangeEventHandler()
    {
    	allJoynLSFClient.initialize();
    }
       
	@SubscribeEvent
	public void onTick(RenderTickEvent event)
	{
		if (Minecraft.getMinecraft().thePlayer == null) {
			return;
		}
		if (event.phase != TickEvent.Phase.START) {
			return;
		}
		
		long ms = System.currentTimeMillis();
		if (!refreshNeeded && (ms - ms0) < elapse) {
			return;
		}
		ms0 = ms;
	
		EntityPlayerSP thePlayer = Minecraft.getMinecraft().thePlayer;
		
		float motionLevel = getMotionLevel(thePlayer);
		elapse = (int)((1.0F-motionLevel) * (MAX_ELAPSE-MIN_ELAPSE) + MIN_ELAPSE);

		float hsb[] = LightLevelHelper.getColorHSB(thePlayer);
	
		// Update only brightness if we are refreshing very frequently
		int n = (elapse > MAX_ELAPSE/2) ? 0 : hsb.length-1;
		
		for (; n<hsb.length; n++) {
			
			if (refreshNeeded || !compareColorElement((long)this.colorHSB[n], (long)hsb[n])) {
				this.colorHSB[n] = hsb[n];
				allJoynLSFClient.setUInt32Property(stringHSB[n], (long)this.colorHSB[n]);
				try {
					Thread.sleep(LSF_SETPROP_DELAY);
				} catch (InterruptedException e) {				
				}
			}
		}
		refreshNeeded = false;
		
		RenderEventHandler.drawText(String.format("Light: %.1f%%", this.colorHSB[2]*100F/LightLevelHelper.MAX_UINT32));
	}
	
	private boolean compareColorElement(long arg1, long arg2)
	{
		// Ignore subtle difference
		final long op = 0xFFFF0000;
		return ((arg1 & op) == (arg2 & op));
	}
	
	/// Return: 0.0 to 1.0
	private float getMotionLevel(EntityPlayerSP thePlayer)
	{
		final float MAX_VELOCITY = 1.0F;
		final float MIN_VELOCITY = 0.08F;
		
		Entity entity;
		if (thePlayer.isRiding()) {
			entity = thePlayer.ridingEntity;
		} else {
			entity = thePlayer;
		}
		
		float distance = (float)Math.sqrt(entity.motionX*entity.motionX + entity.motionY*entity.motionY + entity.motionZ*entity.motionZ);				
		distance = Math.max(MIN_VELOCITY, Math.min(MAX_VELOCITY, distance));
		
		return (distance-MIN_VELOCITY)/(MAX_VELOCITY-MIN_VELOCITY);
	}
}
